@@ -56,6 +56,7 @@ before "proxy-server" and add the following filter in the file:
     password = a_big_secret
     interface = public
 """
+import datetime
 import functools
 import logging
 
@@ -66,7 +67,6 @@ from keystoneclient.v3 import client as ks_client
 from oslo_config import cfg
 import oslo_messaging
 from oslo_utils import strutils
-from oslo_utils import timeutils
 from pycadf import event as cadf_event
 from pycadf.helper import api
 from pycadf import measurement as cadf_measurement
@@ -79,7 +79,7 @@ import threading
 
 from ceilometermiddleware import ksa_adapter
 
-_LOG = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 def list_from_csv(comma_separated_str):
@@ -97,8 +97,8 @@ def _log_and_ignore_error(fn):
         try:
             return fn(*args, **kwargs)
         except Exception as e:
-            _LOG.exception('An exception occurred processing '
-                           'the API call: %s ', e)
+            LOG.exception('An exception occurred processing '
+                          'the API call: %s ', e)
     return wrapper
 
 
@@ -163,7 +163,7 @@ class Swift(object):
             oslo_messaging.get_transport(cfg.CONF, url=conf.get('url')),
             publisher_id='ceilometermiddleware',
             driver=conf.get('driver', 'messagingv2'),
-            topic=conf.get('topic', 'notifications'))
+            topics=[conf.get('topic', 'notifications')])
 
         self.metadata_headers = [h.strip().replace('-', '_').lower()
                                  for h in conf.get(
@@ -174,7 +174,7 @@ class Swift(object):
         if self.reseller_prefix and self.reseller_prefix[-1] != '_':
             self.reseller_prefix += '_'
 
-        _LOG.setLevel(getattr(logging, conf.get('log_level', 'WARNING')))
+        LOG.setLevel(getattr(logging, conf.get('log_level', 'WARNING')))
 
         # NOTE: If the background thread's send queue fills up, the event will
         #  be discarded
@@ -191,15 +191,13 @@ class Swift(object):
             if Swift.event_queue is None:
                 send_queue_size = int(conf.get('send_queue_size', 1000))
                 Swift.event_queue = queue.Queue(send_queue_size)
-                Swift.event_sender = SendEventThread(self._notifier)
-                Swift.event_sender.start()
-                _LOG.debug('Started sender thread')
+                self.start_sender_thread()
             Swift.threadLock.release()
 
     def _get_ignore_projects(self, conf):
         if 'auth_type' not in conf:
-            _LOG.info("'auth_type' is not set assuming ignore_projects are "
-                      "only project uuid.")
+            LOG.info("'auth_type' is not set assuming ignore_projects are "
+                     "only project uuid.")
             return list_from_csv(conf.get('ignore_projects'))
 
         if 'ignore_projects' in conf:
@@ -241,7 +239,7 @@ class Swift(object):
             name_or_id = name_or_id.decode('utf-8', 'strict')
         projects = client.projects.list(name=name_or_id)
         if not projects:
-            _LOG.warning("fail to find project '%s' in keystone", name_or_id)
+            LOG.warning("fail to find project '%s' in keystone", name_or_id)
         return [p.id for p in projects]
 
     def __call__(self, env, start_response):
@@ -320,7 +318,7 @@ class Swift(object):
         except ValueError:
             return
 
-        now = timeutils.utcnow().isoformat()
+        now = datetime.datetime.utcnow().isoformat()
 
         resource_metadata = {
             "path": path,
@@ -369,12 +367,21 @@ class Swift(object):
         if self.nonblocking_notify:
             try:
                 Swift.event_queue.put(event, False)
-                _LOG.debug('Event %s added to send queue', event.id)
+                if not Swift.event_sender.is_alive():
+                    Swift.threadLock.acquire()
+                    self.start_sender_thread()
+                    Swift.threadLock.release()
+
             except queue.Full:
-                _LOG.warning('Send queue FULL: Event %s not added', event.id)
+                LOG.warning('Send queue FULL: Event %s not added', event.id)
         else:
             Swift.send_notification(self._notifier, event)
 
+    def start_sender_thread(self):
+        Swift.event_sender = SendEventThread(self._notifier)
+        Swift.event_sender.daemon = True
+        Swift.event_sender.start()
+
     @staticmethod
     def send_notification(notifier, event):
         notifier.info({}, 'objectstore.http.request', event.as_dict())
@@ -385,19 +392,18 @@ class SendEventThread(threading.Thread):
     def __init__(self, notifier):
         super(SendEventThread, self).__init__()
         self.notifier = notifier
-        self.daemon = True
 
     def run(self):
         """Send events without blocking swift proxy."""
         while True:
             try:
-                _LOG.debug('Wait for event from send queue')
+                LOG.debug('Wait for event from send queue')
                 event = Swift.event_queue.get()
-                _LOG.debug('Got event %s from queue - now send it', event.id)
+                LOG.debug('Got event %s from queue - now send it', event.id)
                 Swift.send_notification(self.notifier, event)
-                _LOG.debug('Event %s sent.', event.id)
-            except Exception:
-                _LOG.exception("SendEventThread loop exception")
+                LOG.debug('Event %s sent.', event.id)
+            except BaseException:
+                LOG.exception("SendEventThread loop exception")
 
 
 def filter_factory(global_conf, **local_conf):
